/**********************************************************************
 * Copyright (c) 2011 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors: 
 * IBM - Initial API and implementation
 **********************************************************************/

package org.eclipse.tptp.platform.jvmti.client.internal.launcher;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.hyades.models.hierarchy.TRCAgent;
import org.eclipse.hyades.models.trace.TRCMemoryStatsManager;
import org.eclipse.hyades.models.trace.TRCProcess;
import org.eclipse.hyades.trace.internal.ui.TraceConstants;
import org.eclipse.hyades.trace.ui.UIPlugin;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.tptp.platform.jvmti.client.internal.TIMessages;
import org.eclipse.tptp.platform.jvmti.client.internal.controlproviders.TIAgentControlProvider;

public class MemoryStatsManager {

	// Constants
		
	private final static int FULL_METH_INVOC_ENTRY_SIZE = 201; // (was 170 then 136)
	private final static int AG_METH_INVOC_ENTRY_SIZE = 166; // (was 120)
	private final static int HEAP_ENTRY_SIZE = 227; // (was 104+64+16)
	
	// Manager variables
	
	private List<TRCProcess> _monitoredProcesses = new ArrayList<TRCProcess>();
	private List<TRCAgent> _monitoredAgents = new ArrayList<TRCAgent>();

	private MemStatsManThread _thread = null;
	
	/** Whether or not the profiling warning has been displayed; we only display this at most once per session. */
	private boolean _heapWarningIssued = false;
	
	
	// Singleton instance variable
	private static MemoryStatsManager _instance = new MemoryStatsManager();
	
	
	private MemoryStatsManager() {
	}
	
	public static MemoryStatsManager getInstance() {
		synchronized(_instance) {
			if(_instance._thread == null) {
				_instance._thread = _instance.new MemStatsManThread();
				_instance._thread.start();
			}
		}
		return _instance;
	}

	public void monitorAgent(TRCAgent a) {
		synchronized(_instance) {
			
			if(a.getProcess() != null && a.getProcess() instanceof TRCProcess) {
				
				if(!_monitoredProcesses.contains(a.getProcess())) { 
					_monitoredProcesses.add((TRCProcess)a.getProcess());
				}
				
			} else {
			
				if(a != null && !_monitoredAgents.contains(a)) {
					_monitoredAgents.add(a);
				}
			}
		}
	}
	
	class MemStatsManThread extends Thread {
		boolean _threadRunning = true;

		/** Whether or not to display the warning, once the necessary conditions are met */
		boolean _reportWarning = false;
		
		public MemStatsManThread() {
			super(MemStatsManThread.class.getName());
			setDaemon(true);
			
			// We read this value from preferences
			_reportWarning = UIPlugin.getDefault().getPreferenceStore().getBoolean(TraceConstants.HEAP_WARNING_PREFERENCE);			
		}
		
		/** Check the contents of the workbench heap to determine if our profiling agent needs to be
		 * paused, in addition to issuing a warning to the user about profiler data usage.   
		 * (Please only call this within a synchronized block holding _instance) */
		private void checkWorkbenchHeap() {
			Runtime r = Runtime.getRuntime();
			float heapPercentUsed = (float)r.totalMemory() / (float)r.maxMemory();
			
			// If the overall heap use is greater than 95%, and also ensure we haven't already given this warning already
			if(heapPercentUsed > 0.95 && !_heapWarningIssued && _reportWarning) {
				
				// Calculate the total workbench heap used by all our profiling data
				long totalBytesUsedByProcesses = 0;
				float ratioUsedByProcesses = 0;
				boolean areAnyAgentsActive = false; // Whether or not any of the agents we are monitoring are actually active
				
				for(TRCProcess p : _monitoredProcesses) {
					long processHeapContrib = 0;
					
					TRCMemoryStatsManager m = p.getProcessMemStats(); 
					if(m != null) {
						processHeapContrib += m.getNumFullMethInvocObjs() * FULL_METH_INVOC_ENTRY_SIZE;
						processHeapContrib += m.getNumAggMethInvocObjs() * AG_METH_INVOC_ENTRY_SIZE;
						processHeapContrib += m.getNumHeapObjs() * HEAP_ENTRY_SIZE;
					}
					
					totalBytesUsedByProcesses += processHeapContrib;
					
					if(p.getAgent() != null && p.getAgent().getAgentProxy() != null && 
							p.getAgent().getAgentProxy().isAttached()) {
						areAnyAgentsActive = true;
					}
					
				}
				
				// How much of the total heap is our profiling data taking 
				ratioUsedByProcesses = ((float)totalBytesUsedByProcesses / (float)r.totalMemory()); 
				
				// If our profiling data is more than 50%, then pause and issue a warning
				// (only do this if the profiling agents are currently active, though)
				if(ratioUsedByProcesses > 0.5 && areAnyAgentsActive) {
					_heapWarningIssued = true;
					
					boolean pauseSuccess = true;
					try {
						// Pause all active profiling agents
						TIAgentControlProvider.TIAgentStateModifier.getInstance().pauseMonitoring();
					} catch (CoreException e) {
						pauseSuccess = false;
					}
					
					// If the pause succeeded, display the warning
					if(pauseSuccess) {
						final Display d = Display.getDefault();
						d.asyncExec(new Runnable() {
							public void run() {
								final String title = TIMessages.PROFILER_MEM_WARNING_TITLE;
								final String message = TIMessages.PROFILER_MEM_WARNING_MSG; 

								MessageDialog.openWarning(d.getActiveShell(), title, message);
							}
						});
					}
				}
				
			}			
		}
		
		public void run() {
			
			while(_threadRunning) {
				
				synchronized(_instance) {

					// Look for processes from our monitored agents
					for(Iterator<TRCAgent> it = _monitoredAgents.iterator(); it.hasNext();) {
						TRCAgent a = it.next();
						if(a.getProcess() != null) {
							if(a.getProcess() instanceof TRCProcess) {
								_monitoredProcesses.add((TRCProcess)a.getProcess());
							}
							// If the agent has a process, we no longer need to monitor it
							it.remove();
						}
					}
					
					// Check the amount of memory consumed by our profiling data, to see if it meets
					// the warning threshold.
					checkWorkbenchHeap();
					
				}
				
				// Sleep 10 seconds between checks
				try {
					Thread.sleep(10 * 1000);
				} catch (InterruptedException e) {
					_threadRunning = false;
				}
				
			}
			
		}
	}
}