/*******************************************************************************
 * Copyright (c) 2005, 2012 Intel Corporation, IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Hoang M Nguyen, Intel - TPTP OS Calls Windows specific implementation
 * $Id: TPTPOSCalls_win.c,v 1.11 2007-04-13 16:05:43 jkubasta Exp $
 *******************************************************************************/ 


#ifdef _WIN32     // Windows-specific

#include <stdarg.h>
#include <stdio.h>
#include "TPTPOSCalls.h"
#include <string.h>
#include <wchar.h>

const char OS_SPECIFIC_NAME_SEPARATOR = '\\' ;

/**
 *********************************************************
 *
 * @brief
 *    Return a global unique id string (GUID or UUID)
 *
 * @note
 *    This requires to link with Windows "rpcrt4.lib" library.
 *    The caller is responsible to free the returned string using free() function
 *
 *********************************************************/

char * getGlobalUniqueId()
{
   char *pTokenStr = NULL ;

   UUID  token ;

   int rc = UuidCreate(&token) ;

   unsigned char * pIdStr = NULL ;

   rc = UuidToString(&token, &pIdStr) ;

   if (pIdStr != NULL)
   {
	   pTokenStr = strdup(pIdStr) ;
	   
	   RpcStringFree(&pIdStr) ;
   } 

   return ( pTokenStr ) ;
}

/**
 *********************************************************
 *                                                       
 * @brief                                                
 *    convenient way to compare string ignoring case
 *    and is still platform independent.
 *
 *********************************************************/
 
int compareIgnoreCase(const char *pStr1, const char *pStr2)
{
	return ( stricmp(pStr1, pStr2) ) ;
}


/**
 *********************************************************
 *                                                       
 * @brief                                                
 *    load a given DLL name
 *
 *********************************************************/

void * loadTheModule(const char *pLibName)
{
	int rc = 0 ;
	HINSTANCE  dllHolder = NULL ;

	//printf("load this library: %s...\n", pLibName) ;

	dllHolder = LoadLibrary( pLibName ) ;

	return (void *) dllHolder ;
}


/**
 *********************************************************
 *                                                       
 * @brief                                                
 *    Start a new running thread and execute the given function
 *       in that running thread.
 *
 * @return
 *    0 - Success
 *    Nonzero - Error
 *
 *********************************************************/

int  startNewThread(RUN_FUNC_TYPE  pFunc, void * pParmBlock, TID *pThreadId, HANDLE *pThreadHandle)
{
	int    rc = 0 ;

	*pThreadHandle = CreateThread(NULL, 0, pFunc, (PVOID) pParmBlock, 0, pThreadId);

	if (*pThreadHandle == NULL)
		rc = 1 ;

	return ( rc ) ;
}

int  tptpStartThread(RUN_FUNC_TYPE  pFunc, void * pParamBlock, TID *pThreadId, HANDLE *pThreadHandle)
{
	return startNewThread (pFunc, pParamBlock, pThreadId, pThreadHandle);
}

/**
 *********************************************************
 *                                                       
 * @brief                                                
 *    Return the process id of the current running process
 *
 *********************************************************/

PID  getCurrentlyRunningProcessId()
{
	return ( GetCurrentProcessId() ) ;
}

/**
 *********************************************************
 *                                                       
 * @brief                                                
 *    Return the executable name of the current running process.
 *    Caller must free the allocated memory for the name string returned.
 *
 *********************************************************/

char *getExecutableName()
{
	char *ptr = NULL;
	char *commandLine = GetCommandLine();
	char *exeName = NULL;
	int len = 0;

	if (!commandLine) return NULL;

	/* Strip off any arguments that following the command name */
	ptr = commandLine;
	len = 0;
	while (*ptr!='\0' && *ptr!=' ' && *ptr!='\t' && *ptr!='\n')
	{
		ptr++;
		len++;
	}

	if (len < 1) return NULL;
	exeName = (char *)tptp_malloc(len +1);
	strncpy(exeName, commandLine, len);
	exeName[len] = '\0';
	return exeName;

}

/**
 *********************************************************
 *                                                       
 * @brief                                                
 *    Return the process id of the current running process
 *
 *********************************************************/

TID  getCurrentlyRunningThreadId()
{
	return ( GetCurrentThreadId() ) ;
}

/**
 *********************************************************
 *                                                       
 * @brief                                                
 *    Print out the current system error message
 *
 *********************************************************/
void printCurrentSysError()
{
		LPVOID lpMsgBuf;
		FormatMessage( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM | 
			FORMAT_MESSAGE_IGNORE_INSERTS,
			NULL,
			GetLastError(),
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(LPTSTR) &lpMsgBuf,
			0,
			NULL 
		);

		printf( (LPTSTR)lpMsgBuf );

		// Free the buffer.
		LocalFree( lpMsgBuf );
}


/**
 *********************************************************
 *                                                       
 * @brief                                                
 *    Put the current thread in sleep mode
 *    for a given amount of time (in milliseconds)
 *
 *********************************************************/
void  tptpSleep(int milliseconds)
{
	Sleep(milliseconds) ;
}

static wchar_t *to_wide( UINT cp, const char *s ) {

	wchar_t *wide = NULL;
	int size = 0;
	
	size = MultiByteToWideChar( cp, MB_ERR_INVALID_CHARS, s, -1, NULL, 0 );
	if( !size )
		return NULL;

	wide = malloc( size * sizeof(wchar_t) );
	if( !wide )
		return NULL;

	if( !MultiByteToWideChar( cp, MB_ERR_INVALID_CHARS, s, -1, wide, size) ) {
		free( wide );
		return NULL;
	}

	return wide;

}

int tptp_fclose( FILE* fp ) {

	return fclose( fp );

}

int tptp_feof( FILE *fp ) {
	return feof( fp );
}

int tptp_ferror( FILE *fp ) {
	return ferror( fp );
}

int tptp_fflush( FILE *fp ) {
	return fflush( fp );
}

char *tptp_fgets( char *buf, int max, FILE* fp ) {
	return fgets( buf, max, fp );
}

/**
 *********************************************************
 *
 * @brief
 *    An implementation of fopen that correctly handles 
 *    paths exceeding MAX_PATH (259) characters.
 *
 *********************************************************/
FILE *tptp_fopen( const char *filename, const char *mode ) {

	char cwd[ 32768 ];
	char absolutePath[ 32768 ];
	char path[ 32768 ];
	char *slash, *prevprev, *prev, *start, *end;
	size_t pathLength;
	wchar_t *widePath = NULL, *wideMode = NULL;
	FILE *fp = NULL;

	pathLength = strlen(filename);
	if( pathLength+1 > sizeof(path) )
		return NULL;
	strcpy( path, filename );

	if( !GetCurrentDirectory( sizeof(cwd), cwd ) )
		return NULL;

	// First, convert / to \ (extended length paths may not contain /'s)
	slash = strchr( path, '/' );
	while( slash ) {
		*slash = '\\';
		slash = strchr( slash+1, '/' );
	}

	// Convert to absolute path //////////////////////////////////////////////
	
	// Some clever chap has already given us an extended-length absolute path? Say no more!
	if( pathLength > strlen("\\\\?\\") 
	    && (   0 == memcmp(path, "\\\\?\\", strlen("\\\\?\\")) 
	        || 0 == memcmp(path, "\\\\.\\", strlen("\\\\.\\"))) ) {

		if( pathLength+1 > sizeof(absolutePath) )
			return NULL;
		strcpy( absolutePath, path );

	}
	// Drive qualified ..
	else if( pathLength > 3 && isalpha(path[0]) && ':' == path[1] ) {

		// .. absolute path: X:\dir\...
		if( '\\' == path[2] ) {

			if( pathLength+1 > sizeof(absolutePath) )
				return NULL;

			strcpy( absolutePath, path );

		}
		// .. relative path: X:dir\...
		else {

			char drive[] = "X:";
			char drivewd[ 32768 ];

			drive[0] = path[0];
			SetCurrentDirectory( drive );
			
			if( !GetCurrentDirectory( sizeof(drivewd), drivewd ) )
				return NULL;

			if( strlen(drivewd)+1 + pathLength-2 + 1 > sizeof(absolutePath) )
				return NULL;

			strcpy( absolutePath, drivewd );
			strcat( absolutePath, "\\" );
			strcat( absolutePath, path+2 );

			SetCurrentDirectory( cwd );

		}

	}
	// A UNC: \\server\...
	else if( pathLength > strlen("\\\\") && 0 == memcmp(path, "\\\\", strlen("\\\\")) ) {

		if( strlen("\\\\?\\UNC\\") + pathLength-2 + 1 > sizeof(absolutePath) )
			return NULL;

		strcpy( absolutePath, "\\\\?\\UNC\\" );
		strcat( absolutePath, path+2 );

	}
	// Drive absolute: \dir1\...
	else if( pathLength > 1 && '\\' == path[0] ) {

		if( strlen("X:") + strlen(path) + 1 > sizeof(absolutePath) )
			return NULL;

		strncpy( absolutePath, cwd, strlen("X:") );
		strcpy( absolutePath + strlen("X:"), path );

	}
	// Relative to cwd: dir1\...
	else {

		if( strlen(cwd) + 1 + pathLength + 1 > sizeof(absolutePath) )
			return NULL;

		strcpy( absolutePath, cwd );
		strcat( absolutePath, "\\" );
		strcat( absolutePath, path );

	}

	// De-relative-ize the path (collapse .\ and ..\ segments)
	pathLength = strlen( absolutePath );
	prevprev = NULL;                      // previous of previous segment
	prev  = NULL;                         // previous segment
	
	// start of current segment
	if( 0 == memcmp( "\\\\?\\", absolutePath, strlen("\\\\?\\") )
	 || 0 == memcmp( "\\\\.\\", absolutePath, strlen("\\\\.\\") ) )
		start = strchr( absolutePath + strlen("\\\\?\\") + 1, '\\' );
	else
		start = strchr( absolutePath, '\\' ); 

	if( start )                           // end of current segment
		end = strchr( start+1, '\\' );
	else
		end = NULL;
	while( end ) {

		// Collapse '\..' and move up one directory level
		if( 0 == memcmp( "\\..", start, max((int)strlen("\\.."), end - start) ) ) {

			// If there is no previous segment, we have a mal-formed path.
			if( !prev )
				return NULL;

			memmove( prev, end, (absolutePath + pathLength) - end + 1 );

			end      = strchr( prev+1, '\\' );
			start    = prev;
			prev     = prevprev;
			while( prevprev > absolutePath && '\\' != *(--prevprev) )
				;
			if( prevprev == absolutePath )
				prevprev = NULL;

		// Collapse '.\' or '\\'
		} else if( 0 == memcmp( "\\.", start, max((int)strlen("\\."), end - start) )
		        || 0 == memcmp( "\\\\", start, max((int)strlen("\\\\"), end - start) ) ) {

			memmove( start, end, (absolutePath + pathLength) - end + 1 );
			end   = strchr( start+1, '\\' );

		// Advance to the next path segment
		} else {

			prevprev = prev;
			prev     = start;
			start    = end;
			end      = strchr( start+1, '\\' );

		}

	}

	// Prepend the extended-length path prefix if necessary
	if( 0 != memcmp(absolutePath, "\\\\?\\", strlen("\\\\?\\"))
	 && 0 != memcmp(absolutePath, "\\\\.\\", strlen("\\\\.\\")) ) {

		if( strlen(absolutePath) + strlen("\\\\?\\") + 1 > sizeof(path) )
			return NULL;

		strcpy( path, "\\\\?\\" );
		strcat( path, absolutePath );

	}
	else
		strcpy( path, absolutePath );

	// Finally, convert to wchar_t ///////////////////////////////////////////

	widePath = to_wide( CP_ACP, path );
	if( !widePath )
		goto done;

	wideMode = to_wide( CP_ACP, mode );
	if( !wideMode )
		goto done;

	fp = _wfopen( widePath, wideMode );

done:
	if( widePath )
		free( widePath );
	if( wideMode )
		free( wideMode );

	return fp;

}

int tptp_fprintf( FILE *fp, const char *fmt, ... ) {

	int ret;

	va_list argv;

	va_start( argv, fmt );
	ret = vfprintf( fp, fmt, argv );
	va_end( argv );

	return ret;

}

size_t tptp_fread( void *dst, size_t el, size_t count, FILE *fp ) {
	return fread( dst, el, count, fp );
}

void tptp_frewind( FILE *fp ) {
	rewind( fp );
}

int tptp_fseek( FILE *fp, long ofs, int origin ) {
	return fseek( fp, ofs, origin );
}

long tptp_ftell( FILE *fp ) {
	return ftell( fp );
}

size_t tptp_fwrite( const void *src, size_t el, size_t count, FILE *fp ) {
	return fwrite( src, el, count, fp );
}

#endif   // end-of-Windows-specific

