/*****************************************************************************
 * Copyright (c) 1997, 2012 Intel Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Intel Corporation - Initial API and implementation
 *
 * $Id$ 
 *****************************************************************************/

#include "ThreadInfoManager.h"
#include "LogAssert.h"
#include "IJVM.h"
#include "EventManager.h"
#include "MVector.h"
#include "JpiGlobals.h"

using namespace Martini::JPI;
using namespace Martini::MPI;
using namespace Martini::Infrastructure;
using namespace Martini::OSA;
using namespace Martini::RTUtil;

typedef MVector<TId> TThreadsVector;
typedef MVector<STlsThreadInfo*> TThreadInfoVector;

//////////////////////////////////////////////////////////////////////
// CThreadInfoManager implementation

CThreadInfoManager::CThreadInfoManager()
: m_ptlsThreadInfo(NULL), m_pEventManager(NULL),
    m_pThreadIdAllocator(NULL), m_pVMInterface(NULL)
{

}

CThreadInfoManager::~CThreadInfoManager()
{
    if (m_ptlsThreadInfo)
    {
        m_ptlsThreadInfo->Destroy();
    }
    //TODO: deallocate all information from internal hash tables, and delete all
    //      global weak thread references stored in the database
}

TResult CThreadInfoManager::Init(CEventManager *pEventManager)
{
    m_pEventManager = pEventManager;
    m_pVMInterface = CJpiGlobals::Instance()->pJvmInterface;

    // Initialize the Thread Local Storage index for storing thread information
    if (!m_ptlsThreadInfo)
    {
        m_ptlsThreadInfo = CreateThreadLocalStorage();
        if (!m_ptlsThreadInfo)
        {
            MARTINI_ERROR("CThreadInfoManager", "Error creating thread local storage");
            return MRTE_ERROR_OSA_FAILURE;
        }
    }

    // Initialize the Thread Id Allocator
    if (!m_pThreadIdAllocator)
    {
        m_pThreadIdAllocator = new CIdAllocator(1);
    }
    
    // Initialize hash tables
    m_JniEnv2MPIIdMap.Init(false, 0, 20);
    m_ThreadMPIId2InfoMap.Init(false, 0, 20);

    return MRTE_RESULT_OK;
    
}

STlsThreadInfo* CThreadInfoManager::GetOrConstructTlsInfo() {

	STlsThreadInfo * tlsInfo = GetTlsInfo();

	if( !tlsInfo ) {
		tlsInfo = new STlsThreadInfo;
        m_ptlsThreadInfo->SetValue( tlsInfo );
	}

	return tlsInfo;

}

TResult CThreadInfoManager::ThreadStart(TId *pNewThreadId, 
                                        const JNIEnv *pJniEnv, 
                                        jthread threadRef,
                                        bool bGenerateEvent,
                                        const char *szName, 
                                        const char *szGroupName, 
                                        const char *szParentGroupName)
{
    if (NULL == pJniEnv)
    {
        return MRTE_ERROR_NULL_PTR;
    }

    jthread threadWeakGlobalRef;
    if (NULL == threadRef)
    {
        threadWeakGlobalRef = NULL;
    }
    else
    {
        JNIEnv* pCurrentJniEnv;
        TResult res = m_pVMInterface->GetJNIEnv(&pCurrentJniEnv);
        if (MRTE_FAILED(res))
        {
            MARTINI_INFORMATIVE("CThreadInfoManager", 0, false, 
                "failed to obtain JNI environment of the calling thread");
            return MRTE_ERROR_FAIL;
        }
        threadWeakGlobalRef = pCurrentJniEnv->NewWeakGlobalRef(threadRef);
        if (NULL == threadWeakGlobalRef)
        {
            MARTINI_ERROR("CThreadInfoManager", 
                "failed to create a weak global reference for the thread");
            return MRTE_ERROR_FAIL;
        }
    }

    TId threadId = m_JniEnv2MPIIdMap.Get((UIOP)pJniEnv);
    STlsThreadInfo *pTlsThreadInfo = GetOrConstructTlsInfo();
    bool bNewThreadInfo = false;

    if (0 == threadId)
    {
        // Create new database entries for this thread
        bNewThreadInfo = true;

        threadId = m_pThreadIdAllocator->AllocateId();

        pTlsThreadInfo->rwLock->EnterWriter();

        // Fill in the STlsThreadInfo structure
        pTlsThreadInfo->threadId = threadId;
        pTlsThreadInfo->threadInfo.szName = szName;
        pTlsThreadInfo->threadInfo.szGroupName = szGroupName;
        pTlsThreadInfo->threadInfo.szParentGroupName = szParentGroupName;
        pTlsThreadInfo->threadState = STlsThreadInfo::ALIVE;
        pTlsThreadInfo->threadType = STlsThreadInfo::JAVA;
        pTlsThreadInfo->pJniEnv = pJniEnv;
        pTlsThreadInfo->wgrThread = threadWeakGlobalRef;

        m_JniEnv2MPIIdMap.Set((UIOP)pJniEnv, threadId);
        m_ThreadMPIId2InfoMap.Set(threadId, pTlsThreadInfo);

        pTlsThreadInfo->rwLock->LeaveWriter();

        MARTINI_INFORMATIVE2("ThreadInfoManager", 5, false, 
            "JNI env %p is associated with new thread id %u", 
            pJniEnv, threadId);
    }
    else
    {
        if (NULL == pTlsThreadInfo)
        {
            MARTINI_ERROR("CThreadInfoManager", "failed to allocate/retrieve thread information");
            return MRTE_ERROR_FAIL;
        }

	    pTlsThreadInfo->rwLock->EnterWriter();

        // Update thread JNI reference if needed
        if (0 == pTlsThreadInfo->wgrThread)
        {
            pTlsThreadInfo->wgrThread = threadWeakGlobalRef;
        }
        pTlsThreadInfo->pJniEnv = pJniEnv;
        
        // Update thread information if needed
		pTlsThreadInfo->threadId = threadId;
        if (NULL == pTlsThreadInfo->threadInfo.szName && szName != NULL)
        {
            bNewThreadInfo = true;
            pTlsThreadInfo->threadInfo.szName = szName;
            pTlsThreadInfo->threadInfo.szGroupName = szGroupName;
            pTlsThreadInfo->threadInfo.szParentGroupName = szParentGroupName;
        }

        pTlsThreadInfo->rwLock->LeaveWriter();

    }

    *pNewThreadId = pTlsThreadInfo->threadId;

    // Generate the Thread Start event if needed. The event will be generated only if it
    // was not generated before and if thread information is available.
    if (bGenerateEvent && bNewThreadInfo)
    {
        NotifyThreadStartEvent(pTlsThreadInfo);
    }
    
    return MRTE_RESULT_OK;

}


TResult CThreadInfoManager::UpdateCurrentThreadTls(TId threadId)
{
    STlsThreadInfo *pTlsThreadInfo = m_ThreadMPIId2InfoMap.Get(threadId);
    if (NULL == pTlsThreadInfo)
    {
        return MRTE_ERROR_FAIL;
    }
    
    m_ptlsThreadInfo->SetValue(pTlsThreadInfo);
    return MRTE_RESULT_OK;
}

TResult CThreadInfoManager::ThreadEnd(const JNIEnv *pJniEnv)
{
    if (NULL == pJniEnv)
    {
	    MARTINI_INFORMATIVE("CThreadInfoManager", 0, false, 
	                        "NULL JNIEnv* in ThreadEnd()" );
	    return MRTE_ERROR_NULL_PTR;
    }

    TId threadId = m_JniEnv2MPIIdMap.Del( (UIOP)pJniEnv );
    if (0 == threadId)
    {
	    MARTINI_INFORMATIVE("CThreadInfoManager", 0, false, 
            "failed to convert an existing JNI pointer to thread id");
        return MRTE_ERROR_FAIL;
    }

    STlsThreadInfo *pTlsThreadInfo = m_ThreadMPIId2InfoMap.Get(threadId);
    if (NULL != pTlsThreadInfo)
    {
	    pTlsThreadInfo->rwLock->EnterWriter();
	    
        pTlsThreadInfo->threadState = STlsThreadInfo::TERMINATED;
	    m_ptlsThreadInfo->SetValue(NULL);
	    
    	pTlsThreadInfo->rwLock->LeaveWriter();
    }

    return MRTE_RESULT_OK;

}

int CThreadInfoManager::ThreadEnterJPI()
{

	STlsThreadInfo *tlsInfo = GetOrConstructTlsInfo();
	return atomicInc( &tlsInfo->entryCount );

}

int CThreadInfoManager::ThreadLeaveJPI()
{

	STlsThreadInfo *tlsInfo = GetTlsInfo();
	if( NULL == tlsInfo )
		// thread has terminated, do nothing
		return 0;

	tlsInfo->rwLock->EnterWriter();

	int nesting = atomicDec( &tlsInfo->entryCount );
	STlsThreadInfo::EThreadState threadState = tlsInfo->threadState;

	if( STlsThreadInfo::SUSPENDING == threadState && 0 == nesting ) {

		tlsInfo->threadState = STlsThreadInfo::SUSPENDED;
		tlsInfo->rwLock->LeaveWriter();
		
		// We are leaving the profiler, its safe to suspend
		TResult result = m_pVMInterface->SuspendThread( tlsInfo->wgrThread );

		if( MRTE_FAILED(result) ) {

			MARTINI_INFORMATIVE1( "ThreadInfoManager", 3, false, "deferred suspend of thread %u failed",
			                      (unsigned)tlsInfo->threadId );
			
			tlsInfo->rwLock->EnterWriter();
			tlsInfo->threadState = threadState;
			tlsInfo->rwLock->LeaveWriter();
		}
		else
			MARTINI_DEBUG1( "ThreadInfoManager", 3, "deferred suspend of thread %u complete",
			                      (unsigned)tlsInfo->threadId );
		
	}
	else
		tlsInfo->rwLock->LeaveWriter();
		
	return nesting;
	
}

TId CThreadInfoManager::GetThreadIdAndUpdateTls(const JNIEnv *pJniEnv, jthread thread)
{
    TResult iRes;
    TId threadId = GetThreadId(pJniEnv);
    if (0 == threadId)
    {
        // Get thread information (if possible)
        SThreadInfo threadInfo;
        memset(&threadInfo, 0, sizeof(SThreadInfo));
        if (thread != NULL)
        {
            m_pVMInterface->GetThreadInfo(&threadInfo, thread);
        }
        iRes = ThreadStart(&threadId, pJniEnv, thread, true, threadInfo.szName, 
            threadInfo.szGroupName, threadInfo.szParentGroupName);
        MARTINI_ASSERT("CEventManager", MRTE_SUCCEEDED(iRes), "");
    }
    iRes = UpdateCurrentThreadTls(threadId);
    MARTINI_ASSERT("CEventManager", MRTE_SUCCEEDED(iRes), "");
    return threadId;
}

TResult CThreadInfoManager::GetThreadInfo(SThreadInfo *pThreadInfo, TId threadId)
{
    if (NULL == pThreadInfo)
    {
        return MRTE_ERROR_NULL_PTR;
    }

    STlsThreadInfo *pTlsThreadInfo = m_ThreadMPIId2InfoMap.Get(threadId);
    if (NULL == pTlsThreadInfo)
    {
        return MRTE_ERROR_FAIL;
    }

    pTlsThreadInfo->rwLock->EnterWriter();

    if (NULL == pTlsThreadInfo->threadInfo.szName)
    {
	    pTlsThreadInfo->rwLock->LeaveWriter();
        MARTINI_INFORMATIVE("ThreadInfoManager", 5, false, 
            "Information for a thread was requested before its Thread Start event was sent");
        return MRTE_ERROR_FAIL;
    }
    pThreadInfo->szName = pTlsThreadInfo->threadInfo.szName;
    pThreadInfo->szGroupName = pTlsThreadInfo->threadInfo.szGroupName;
    pThreadInfo->szParentGroupName = pTlsThreadInfo->threadInfo.szParentGroupName;

    pTlsThreadInfo->rwLock->LeaveWriter();
    
    return MRTE_RESULT_OK;
}

void CThreadInfoManager::SetThreadType(const JNIEnv *pJniEnv, 
                                       STlsThreadInfo::EThreadType threadType)
{
    TId threadId = GetThreadId(pJniEnv);
    if (0 != threadId)
    {
        STlsThreadInfo *pTlsThreadInfo = m_ThreadMPIId2InfoMap.Get(threadId);
        if (NULL != pTlsThreadInfo)
        {
	        pTlsThreadInfo->rwLock->EnterWriter();
            pTlsThreadInfo->threadType = threadType;
	        pTlsThreadInfo->rwLock->LeaveWriter();
        }
        else
        {
            MARTINI_INFORMATIVE("ThreadInfoManager", 0, false, 
                "failed to set thread type. Thread information is not available");
        }
    }
    else
    {
        MARTINI_INFORMATIVE("ThreadInfoManager", 0, false, 
            "failed to set thread type. Unknown JNI environment");
    }
}

bool ThreadInfoHashIterationCallback(void *pParameter, 
                                     const TId threadId,
                                     STlsThreadInfo *pTlsThreadInfo)
{
    TThreadsVector *pThreadsVector = (TThreadsVector*)pParameter;
    if (NULL != pTlsThreadInfo)
    {
        pThreadsVector->Push(threadId);
    }
    return true;
}

bool GetInfoForAllThreads(void *pParameter, 
                          const TId threadId,
                          STlsThreadInfo *pTlsThreadInfo)
{
    TThreadInfoVector *pThreadsVector = (TThreadInfoVector*)pParameter;
    if (NULL != pTlsThreadInfo)
    {
        pThreadsVector->Push(pTlsThreadInfo);
    }
    return true;
}

void CThreadInfoManager::UpdateInfo()
{
    TThreadsVector threadsVector;
    m_ThreadMPIId2InfoMap.Iterate(ThreadInfoHashIterationCallback, &threadsVector);

    for (size_t i = 0; i < threadsVector.Size(); ++i)
    {
        TId threadId = threadsVector.GetAt(i);
        STlsThreadInfo *pTlsThreadInfo = m_ThreadMPIId2InfoMap.Get(threadId);
        
        if( NULL == pTlsThreadInfo )
	        continue;

        SThreadInfo threadInfo;
        TResult iRes = m_pVMInterface->GetThreadInfo(&threadInfo, pTlsThreadInfo->wgrThread);
        if (MRTE_SUCCEEDED(iRes))
        {
	        pTlsThreadInfo->rwLock->EnterWriter();
	        
	        // We want to know the existing name if any; if there is no existing name
	        // then we send the thread start event.
	        const char *existingName = pTlsThreadInfo->threadInfo.szName;
	        
	        pTlsThreadInfo->threadInfo.szName = threadInfo.szName;
	        pTlsThreadInfo->threadInfo.szGroupName = threadInfo.szGroupName;
	        pTlsThreadInfo->threadInfo.szParentGroupName = threadInfo.szParentGroupName;
	        
	        pTlsThreadInfo->rwLock->LeaveWriter();
	        
	        // Generate EV_THREAD_START event (again) so that the profiler can get
	        // the thread information.
	        if( NULL == existingName )
		        NotifyThreadStartEvent(pTlsThreadInfo);
        }
    }
    
}

void CThreadInfoManager::VmShutDown()
{
    TThreadsVector threadsVector;
    m_ThreadMPIId2InfoMap.Iterate(ThreadInfoHashIterationCallback, &threadsVector);

    for (size_t i = 0; i < threadsVector.Size(); ++i)
    {
        TId threadId = threadsVector.GetAt(i);
        STlsThreadInfo *pTlsThreadInfo = m_ThreadMPIId2InfoMap.Get(threadId);
        if (NULL == pTlsThreadInfo)
	        continue;

        pTlsThreadInfo->rwLock->EnterReader();
        STlsThreadInfo::EThreadState threadState = pTlsThreadInfo->threadState;
        pTlsThreadInfo->rwLock->LeaveReader();

        if (STlsThreadInfo::ALIVE == threadState)
        {
	        // Generate EV_THREAD_END event for this thread. 
	        // The lock is released during the callback
	        NotifyThreadEndEvent(pTlsThreadInfo);
        }

    }
    
}

void CThreadInfoManager::NotifyThreadStartEvent(const STlsThreadInfo *pTlsThreadInfo)
{
    SJVMData data;
    SEmData mpiData;
    mpiData.u.pJVMData = &data;
    mpiData.dataType = MPI_VM;
    data.dataType = DT_THREAD_START;

    pTlsThreadInfo->rwLock->EnterReader();

    data.u.threadStart.pThreadEnv = pTlsThreadInfo->pJniEnv;
    data.u.threadStart.szName = pTlsThreadInfo->threadInfo.szName;
    data.u.threadStart.szGroupName = pTlsThreadInfo->threadInfo.szGroupName;
    data.u.threadStart.szParentGroupName = pTlsThreadInfo->threadInfo.szParentGroupName;
    data.u.threadStart.threadLocalRef = pTlsThreadInfo->wgrThread;

    pTlsThreadInfo->rwLock->LeaveReader();

    m_pEventManager->NotifyMpiEvent(EV_THREAD_START, &mpiData);
}

void CThreadInfoManager::NotifyThreadEndEvent(const STlsThreadInfo *pTlsThreadInfo)
{
    SJVMData data;
    SEmData mpiData;
    mpiData.u.pJVMData = &data;
    mpiData.dataType = MPI_VM;
    data.dataType = DT_THREAD_END;

    pTlsThreadInfo->rwLock->EnterReader();

    data.u.threadEnd.pThreadEnv = pTlsThreadInfo->pJniEnv;
    data.u.threadEnd.threadLocalRef = pTlsThreadInfo->wgrThread;

    pTlsThreadInfo->rwLock->LeaveReader();

    m_pEventManager->NotifyMpiEvent(EV_THREAD_END, &mpiData);
}

TResult CThreadInfoManager::SuspendAllThreads()
{
    TThreadsVector threadsVector;

	MARTINI_INFORMATIVE("ThreadInfoManager", 3, false,  "[SuspendAllThreads] Suspending all threads");

	m_ThreadMPIId2InfoMap.Iterate(ThreadInfoHashIterationCallback, &threadsVector);

	bool bError = false;
	for (size_t i = 0; !bError && i<threadsVector.Size(); ++i)
	{
		TId threadId = threadsVector.GetAt(i);
		STlsThreadInfo *pTlsThreadInfo = m_ThreadMPIId2InfoMap.Get(threadId);
        if (NULL == pTlsThreadInfo)
	        continue;

        pTlsThreadInfo->rwLock->EnterWriter();

        STlsThreadInfo::EThreadState threadState = pTlsThreadInfo->threadState;
        STlsThreadInfo::EThreadType threadType =  pTlsThreadInfo->threadType;
        jthread wgrThread = pTlsThreadInfo->wgrThread;

        // ignore threads which are external, or are not alive
        if( STlsThreadInfo::EXTERNAL == threadType 
            || STlsThreadInfo::ALIVE != threadState ) {

	        pTlsThreadInfo->rwLock->LeaveWriter();
	        continue;

        }

        pTlsThreadInfo->threadState = STlsThreadInfo::SUSPENDING;

        int entryDepth = atomicInc( &pTlsThreadInfo->entryCount );
        if( entryDepth > 1 ) {

	        // Deferred suspend. See ThreadLeaveJPI().
	        MARTINI_DEBUG2( "ThreadInfoManager", 3, "[SuspendAllThreads] Thread %u is %d levels deep in the profiler, deferring suspend",
	                        (unsigned)pTlsThreadInfo->threadId, 
	                        entryDepth-1 );
	        pTlsThreadInfo->rwLock->LeaveWriter();

        }
        else {
		        
	        pTlsThreadInfo->rwLock->LeaveWriter();
	        TResult iRes = m_pVMInterface->SuspendThread(wgrThread);

	        if( MRTE_SUCCEEDED(iRes) || MRTE_ERROR_ILLEGAL_OPERATION != iRes ) {
		        pTlsThreadInfo->rwLock->EnterWriter();
		        pTlsThreadInfo->threadState = STlsThreadInfo::SUSPENDED;
		        pTlsThreadInfo->rwLock->LeaveWriter();
	        }
	        else {
		        MARTINI_INFORMATIVE2("ThreadInfoManager", 1, false, "[SuspendAllThreads] Failed to suspend thread: %u, reason=%d", 
		                             (unsigned)pTlsThreadInfo->threadId, 
		                             iRes);
		        
		        pTlsThreadInfo->rwLock->EnterWriter();
		        pTlsThreadInfo->threadState = threadState;
		        pTlsThreadInfo->rwLock->LeaveWriter();
		        
		        bError = true;
	        }
	        
        }
        atomicDec( &pTlsThreadInfo->entryCount );

    }
    
    if (bError)
    {
        return MRTE_ERROR_FAIL;
    }
    return MRTE_RESULT_OK;
}

TResult CThreadInfoManager::ResumeAllThreads()
{
    TThreadsVector threadsVector;

	MARTINI_INFORMATIVE("ThreadInfoManager", 3, false,  "[ResumeAllThreads] Resuming all threads");

    m_ThreadMPIId2InfoMap.Iterate(ThreadInfoHashIterationCallback, &threadsVector);
    bool bError = false;
    for (size_t i = 0; !bError && i<threadsVector.Size(); ++i)
    {
        TId threadId = threadsVector.GetAt(i);
        STlsThreadInfo *pTlsThreadInfo = m_ThreadMPIId2InfoMap.Get(threadId);
        if (NULL == pTlsThreadInfo)
	        continue;

        pTlsThreadInfo->rwLock->EnterWriter();

        STlsThreadInfo::EThreadState threadState = pTlsThreadInfo->threadState;

        // If thread is in a deferred suspending, we just wake it up
        if( STlsThreadInfo::SUSPENDING == threadState ) {

	        MARTINI_DEBUG1( "ThreadInfoManager", 3, "[ResumeAllThreads] Cancel deferred-suspend of thread %u", 
	                        (unsigned)pTlsThreadInfo->threadId );
	        
	        pTlsThreadInfo->threadState = STlsThreadInfo::ALIVE;
	        pTlsThreadInfo->rwLock->LeaveWriter();
	        continue;

        }
        
        // change state in preparation
        pTlsThreadInfo->threadState = STlsThreadInfo::ALIVE;
        pTlsThreadInfo->rwLock->LeaveWriter();
       
        TResult iRes = MRTE_RESULT_OK;

        if( STlsThreadInfo::SUSPENDED == threadState ) {

	        m_pVMInterface->ResumeThread(pTlsThreadInfo->wgrThread);

	        // we may have interleaved w/ ThreadLeaveJPI, so try a few times if 
	        // we are being the thread is not suspended.
	        for( int retry=0; retry<10 && MRTE_ERROR_ILLEGAL_OPERATION == iRes; retry++ )
		        iRes = m_pVMInterface->ResumeThread(pTlsThreadInfo->wgrThread);

        }

        if (MRTE_FAILED(iRes))
        {
	        MARTINI_INFORMATIVE2("ThreadInfoManager", 1, false, "[ResumeAllThreads] Failed to resume thread: %u, reason=%d", 
	                             (unsigned)pTlsThreadInfo->threadId,
	                             iRes);
	        
	        // if it didn't work, reset the thread state
	        pTlsThreadInfo->rwLock->EnterWriter();
	        pTlsThreadInfo->threadState = threadState;
	        pTlsThreadInfo->rwLock->LeaveWriter();
	        
	        bError = true;
        }
        else
	        MARTINI_DEBUG1("ThreadInfoManager", 5, "[ResumeAllThreads] Resumed thread: %u", 
	                       (unsigned)pTlsThreadInfo->threadId);

    }
    
    if (bError)
    {
        return MRTE_ERROR_FAIL;
    }
    return MRTE_RESULT_OK;
}

STlsThreadInfo::EThreadType CThreadInfoManager::GetThreadType(const JNIEnv *pJniEnv)
{
    STlsThreadInfo::EThreadType res = STlsThreadInfo::UNKNOWN;

    TId threadId = GetThreadId(pJniEnv);
    if (0 != threadId)
    {
        STlsThreadInfo *pTlsThreadInfo = m_ThreadMPIId2InfoMap.Get(threadId);
        if (NULL != pTlsThreadInfo)
        {
            res = pTlsThreadInfo->threadType;
        }
    }

    return res;
}

STlsThreadInfo::EThreadType CThreadInfoManager::GetThreadType(TId threadId)
{
    STlsThreadInfo::EThreadType res = STlsThreadInfo::UNKNOWN;
    STlsThreadInfo *pTlsThreadInfo = m_ThreadMPIId2InfoMap.Get(threadId);
    if (NULL != pTlsThreadInfo)
    {
        res = pTlsThreadInfo->threadType;
    }
    return res;
}

jthread CThreadInfoManager::GetThreadObject(const JNIEnv *pJniEnv)
{
    jthread res = NULL;

    TId threadId = GetThreadId(pJniEnv);
    if (0 != threadId)
    {
        STlsThreadInfo *pTlsThreadInfo = m_ThreadMPIId2InfoMap.Get(threadId);
        if (NULL != pTlsThreadInfo)
        {
            res = pTlsThreadInfo->wgrThread;
        }
    }

    return res;
}

TId CThreadInfoManager::GetThreadId(jthread thread)
{
    TThreadInfoVector threadInfoVec;
    JNIEnv *pJniEnv;
    TResult res = m_pVMInterface->GetJNIEnv(&pJniEnv);
    if (MRTE_FAILED(res))
    {
        return 0;
    }
    m_ThreadMPIId2InfoMap.Iterate(GetInfoForAllThreads, &threadInfoVec);
    size_t i;
    TId threadId = 0;
    for (i = 0; i < threadInfoVec.Size(); ++i)
    {
        STlsThreadInfo *pInfo = threadInfoVec.GetAt(i);
        if (pJniEnv->IsSameObject(thread, pInfo->wgrThread))
        {
            threadId = pInfo->threadId;
            break;
        }
    }
    return threadId;
}

TResult CThreadInfoManager::GetStack(jvmtiFrameInfo *pFrameBuffer, 
                                     jint *pFrameCount, 
                                     TId threadId, 
                                     jint maxFrameCount)
{
    jthread thread = NULL;
    if (threadId != 0)
    {
        STlsThreadInfo *pTlsInfo = m_ThreadMPIId2InfoMap.Get(threadId);
        if (NULL == pTlsInfo)
        {
             MARTINI_INFORMATIVE1("ThreadInfoManager", 5, false, 
                "[GetStack] unknown thread id %u", threadId);
           return MRTE_ERROR_FAIL;
        }
        thread = pTlsInfo->wgrThread;
    }
    return m_pVMInterface->GetStackTrace(pFrameBuffer, pFrameCount, thread, 0, maxFrameCount);
}

TResult CThreadInfoManager::GetState(EThreadState *pState, TId threadId)
{
    jthread thread = NULL;
    if (threadId != 0)
    {
        STlsThreadInfo *pTlsInfo = m_ThreadMPIId2InfoMap.Get(threadId);
        if (NULL == pTlsInfo)
        {
             MARTINI_INFORMATIVE1("ThreadInfoManager", 5, false, 
                "[GetState] unknown thread id %u", threadId);
           return MRTE_ERROR_FAIL;
        }
        thread = pTlsInfo->wgrThread;
    }
    jint jvmState = 0;
    TResult res = m_pVMInterface->GetThreadState(&jvmState, thread);
    if (MRTE_FAILED(res))
    {
        return res;
    }

    *pState = m_pVMInterface->VmThreadStateToMpiThreadState(jvmState);
    
    return MRTE_RESULT_OK;
}

TResult CThreadInfoManager::GetThreadObject(jthread *pThread, TId threadId)
{
    if (NULL == pThread)
    {
        return MRTE_ERROR_NULL_PTR;
    }

    if (threadId != 0)
    {
        STlsThreadInfo *pTlsInfo = m_ThreadMPIId2InfoMap.Get(threadId);
        if (pTlsInfo != NULL)
        {
            *pThread = pTlsInfo->wgrThread;
        }
        else
        {
            return MRTE_ERROR_FAIL;
        }
    }
    else
    {
        *pThread = NULL; // NULL identifies the current thread in JVMTI
    }
    return MRTE_RESULT_OK;
}
