/*******************************************************************************
 * Copyright (c) 2006, 2013 Intel Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    
 * $Id$
 *
 *******************************************************************************/
package org.eclipse.tptp.platform.execution.client.core.internal;

import java.io.IOException;
import java.net.Socket;
import java.security.Provider;
import java.security.Security;
import java.security.Provider.Service;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.TrustManager;

import org.eclipse.tptp.platform.execution.client.core.INode;
import org.eclipse.tptp.platform.execution.exceptions.LoginFailedException;
import org.eclipse.tptp.platform.execution.exceptions.ReconnectRequestedException;
import org.eclipse.tptp.platform.execution.exceptions.SecureConnectionRequiredException;
import org.eclipse.tptp.platform.execution.exceptions.UntrustedAgentControllerException;

public class SecureConnectionImpl extends ConnectionImpl {
	
	// Security protocol related variables
	String _protocol="SSL";
	String _userProvidedProtocol;
	
    private SSLSocket initSSL (Socket socket) throws IOException {
    	if (socket == null || !socket.isConnected()) return null;
    	
		TrustManager tms[] = null;
		try {
			X509TrustManagerImpl tm = new X509TrustManagerImpl();
			tms = new TrustManager[1];
			tms[0] = tm;
		} catch (Exception e) {}
		
		/* Determine the acceptable protocol */
		String userProvidedProtocol = System.getProperty("SECURITY_SOCKET_PROTOCOL");
		String protocol = getProtocol(userProvidedProtocol);
		
		SSLContext sslCtx;
		try {
			sslCtx = SSLContext.getInstance(protocol);
			sslCtx.init(null, tms, null);
		}
		catch(Exception e) {
			sslCtx = null;
		}
		
		if (sslCtx == null) return null;
		
		String host = socket.getInetAddress().getHostAddress();
		int port = socket.getPort();
		
		SSLSocket sslSocket = (SSLSocket) sslCtx.getSocketFactory().createSocket(socket, host, port, true);
		sslSocket.setUseClientMode(true);

		sslSocket.startHandshake();

		if (sslSocket.getSession() == null) return null;
		
		return sslSocket;
    }
    
    public boolean connect(INode node, ConnectionImpl con) throws IOException, UntrustedAgentControllerException, 
    	ReconnectRequestedException, SecureConnectionRequiredException, LoginFailedException {

    	if (con == null) return false;
    	Socket socket = con.getSocket();
    	if (socket == null || !socket.isConnected()) return false;
    	
    	con.setSocket(null);	// to save socket from garbage collecting
    	
		setSoTimeout(socket.getSoTimeout());
		_node = node;
		_port = socket.getPort();
		inetAddress = socket.getInetAddress();
			
		SSLSocket sslSocket = initSSL(socket);
		if (sslSocket == null) return false;
		
		setSocket (sslSocket);

		init();		// to complete connection establishment
		
		return true;
    }
    
	public int createDataConnection(int direction) throws IOException, SecureConnectionRequiredException {
		Socket dataSock = connectSocket();
		if (dataSock == null) throw new IOException();

		int dataConnectionId = -1;
		boolean securityRequired = false;

		try {
			dataConnectionId = initDataConnection(dataSock, direction);
		} catch (SecureConnectionRequiredException e) {
			securityRequired = true;
		}
		
		if (!securityRequired) return dataConnectionId;
		
		SSLSocket sslDataSock = initSSL(dataSock);
		
		return initDataConnection(sslDataSock, direction);
	}
	
	// Security protocol related code
	/*  
	 * Note: The exact code exists in the following three classes:
	 * 			org.eclipse.tptp.platform.agentcontroller.jsse.SSLFactory
	 * 			org.eclipse.tptp.platform.execution.client.core.internal.SecureConnectionImpl
	 * 			org.eclipse.hyades.internal.execution.local.control.SecurityProtocol
	 * Any changes made to the following methods need to be replicated in all three classes.
	 * 
	 */
	
	public String getProtocol(String userProtocol) {
		_userProvidedProtocol = userProtocol;
		
		if (_userProvidedProtocol!=null) {
			// use user supplied protocol if it is valid
			_protocol = validate(_userProvidedProtocol) ? _userProvidedProtocol : _protocol;
		}
		else {
			_protocol = (isIBMJVM() && validate("SSL_TLSv2")) ? "SSL_TLSv2" : _protocol;
		}
		
		return _protocol;
	}
	
	/**
	 * Checks if the protocol is supported by the JVM used by the agent controller 
	 */
	public boolean validate(String protocol) {
		//ArrayList<String> protocolsList = getJVMSupportedProtocols();
		
		// Check for any of the other valid algorithms
		if(protocol.equalsIgnoreCase("default")) {
			return false;
		}		
		
		Provider[] providers = Security.getProviders();
		for (int p = 0; p < providers.length; p++) {
			Provider provider = providers[p];
			//System.out.println("Provider[" + p + "] = " + providers[p]);
			// see if this provider supports the provided security protocol
			Service service = provider.getService(SSLContext.class.getSimpleName(), protocol);
			if(service != null) {
				return true;
			}
		}
		return false;
	}
	
	public boolean isIBMJVM(){
		if (System.getProperty("java.vm.vendor").indexOf("IBM") >= 0) {
			//String version = System.getProperty("java.version");
			return true;
		}
		return false;
	}
}